deferred class
	AST_NODE

inherit
	HASHABLE
	ANTLR_TRANSFORMATION_UTILITIES
	VIEW_INTERFACE

feature {ANY} -- AST

	parent: AST_NODE assign set_parent

	parentheses_parent: AST_NODE
			-- Returns either the node itself or the outermost surrounding parentheses node.
			-- For `a' in `a + 1', `a' is returned.
			-- For `a' in `(*((a))) + 1', `((a))' is returned.
		do
			from
				Result := Current
			until
				not attached {AST_PARENTHESIZED_EXPRESSION} Result.parent
			loop
				Result := Result.parent
			end
		ensure
			Result /= Void
		end

	non_parentheses_parent: AST_NODE
		require
			parent /= Void
		do
			from
				Result := parent
			until
				not attached {AST_PARENTHESIZED_EXPRESSION} Result
			loop
				Result := Result.parent
			end
		ensure
			Result /= Void and then not attached {AST_PARENTHESIZED_EXPRESSION} Result
		end

	accept (v: AST_VISITOR)
		deferred
		end

	ast_twin: like Current
		deferred
		end

	make_copy (other: like Current)
		deferred
		end

feature {ANY} -- utilities

	c_code: STRING
		local
			v: AST_C_CREATOR
		do
			create v.make
			accept (v)
			Result := v.buffer
		end

	eiffel_code: STRING
		local
			v: AST_EIFFEL_CREATOR
		do
			create v.make
			accept (v)
			Result := v.buffer
		end

feature {AST_NODE}

	set_parent (init_parent: AST_NODE)
		do
			parent := init_parent
		ensure
			parent_set: parent = init_parent
		end

	visit_node (node: AST_NODE; v: AST_VISITOR)
		require
			visitor_exists: v /= Void
		do
			if node /= Void then
				node.accept (v)
			end
		end

	visit_list (list: LIST [AST_NODE]; v: AST_VISITOR)
		require
			visitor_exists: v /= Void
		do
			if list /= Void then
				across list.twin as c loop
					if c.item /= Void then
						c.item.accept (v)
					end
				end
			end
		end

feature {ANY} -- HASHABLE

	hash_code_cache: INTEGER_32

	hash_code: INTEGER_32
		do
			-- don't use object once; crashes finalized :-/
			if hash_code_cache = 0 then
				hash_code_cache := ($Current).hash_code
			end
			Result := hash_code_cache
		end

feature {NONE} -- invariant helpers

	ast_children_from_reflection: LIST [AST_NODE]
		local
			l: TWO_WAY_LIST [ANY]
			internal: INTERNAL
			i: INTEGER
			field_name: STRING
		do
			from
				create l.make
				create internal
				i := internal.field_count (Current)
			until
				i < 1
			loop
				field_name := internal.field_name (i, Current)
				if
					not field_name.is_equal ("parent") and
					not field_name.is_equal ("type_binding") and field_name[1] /= '_' and
					not field_name.is_equal ("declaration_binding") and
					not field_name.is_equal ("orig_c_dimension") and -- AST_ARRAY_TYPE
					not field_name.is_equal ("orig_c_expression") -- AST_ASM_OPERAND
				then
					l.extend (internal.field (i, Current))
				end
				i := i - 1
			end

			from
				create {TWO_WAY_LIST [AST_NODE]}Result.make
				l.start
			until
				l.is_empty
			loop
				if attached {AST_NODE} l.item as node then
					Result.extend (node)
				elseif attached {LIST [ANY]} l.item as list then
					across list as c loop
						l.extend (c.item)
					end
				end
				l.remove
			end
		ensure
			Result /= Void
		end

	ast_children_from_visitor: LIST [AST_NODE]
		local
			v: ACCEPT_CHECK_VISITOR
		do
			create v.make (Current)
			v.collect_children
			Result := v.collected_children
		ensure
			Result /= Void
		end

	is_ast_valid: BOOLEAN
		local
			reflection, visitor: LIST [AST_NODE]
		do
			if check_ast then
				reflection := ast_children_from_reflection
				visitor := ast_children_from_visitor

				-- check that
				-- o all children collected by reflection have `Current' as parent
				-- o the visitor visits all children
				Result := across reflection as c all c.item.parent = Current end and reflection.count = visitor.count
			else
				Result := True
			end
		end

invariant
	is_ast_valid

end
