class
	SOURCE_INFO

create
	make

feature {NONE}

	make (a_source_file: STRING)
		require
			a_source_file /= Void
		do
			orig_source_file := a_source_file
			is_system_source :=
					({PLATFORM}.is_unix and then orig_source_file.count >= 1 and then orig_source_file[1] = '/') or
					({PLATFORM}.is_windows and then orig_source_file.count >= 2 and then orig_source_file[2] = ':')
			is_project_source := not is_system_source
			if is_system_source then
				source_file := adapted_include_file (orig_source_file)
			else
				source_file := orig_source_file
			end
		end

feature {ANY}

	is_system_source: BOOLEAN

	is_project_source: BOOLEAN

	source_file: STRING

	orig_source_file: STRING

feature {NONE}

	adapted_include_file (full_path: STRING): STRING
		require
			full_path /= Void
			{PLATFORM}.is_unix implies (full_path.count >= 1 and then full_path[1] = '/')
			{PLATFORM}.is_windows implies (full_path.count >= 2 and then full_path[2] = ':')
		local
			tmp: INTEGER_32
		do
			    if full_path.is_equal ("/usr/include/bits/dirent.h") then                  Result := "dirent.h"
			elseif full_path.is_equal ("/usr/include/bits/ioctl-types.h") then             Result := "sys/ioctl.h"
			elseif full_path.is_equal ("/usr/include/bits/mathcalls.h") then               Result := "math.h"
			elseif full_path.is_equal ("/usr/include/bits/mathinline.h") then              Result := "math.h"
			elseif full_path.is_equal ("/usr/include/bits/resource.h") then                Result := "sys/resource.h"
			elseif full_path.is_equal ("/usr/include/bits/sigaction.h") then               Result := "signal.h"
			elseif full_path.is_equal ("/usr/include/bits/siginfo.h") then                 Result := "signal.h"
			elseif full_path.is_equal ("/usr/include/bits/sigstack.h") then                Result := "signal.h"
			elseif full_path.is_equal ("/usr/include/bits/socket.h") then                  Result := "sys/socket.h"
			elseif full_path.is_equal ("/usr/include/bits/stat.h") then                    Result := "sys/stat.h"
			elseif full_path.is_equal ("/usr/include/bits/stdio.h") then                   Result := "stdio.h"
			elseif full_path.is_equal ("/usr/include/bits/string2.h") then                 Result := "string.h"
			elseif full_path.is_equal ("/usr/include/bits/termios.h") then                 Result := "termios.h"
			elseif full_path.is_equal ("/usr/include/bits/time.h") then                    Result := "time.h"
			elseif full_path.is_equal ("/usr/include/bits/types.h") then                   Result := "sys/types.h"
			elseif full_path.is_equal ("/usr/include/bits/waitstatus.h") then              Result := "sys/wait.h"
			elseif full_path.starts_with ("/usr/include/atk-1.0/atk/") then                Result := "atk/atk.h"
			elseif full_path.starts_with ("/usr/include/gdk-pixbuf-2.0/gdk-pixbuf/") then  Result := "gdk-pixbuf/gdk-pixbuf.h"
			elseif full_path.starts_with ("/usr/include/glib-2.0/glib/") then              Result := "glib.h"
			elseif full_path.starts_with ("/usr/include/glib-2.0/gobject/") then           Result := "glib-object.h"
			elseif full_path.starts_with ("/usr/include/gtk-2.0/gdk/") then                Result := "gdk/gdk.h"
			elseif full_path.starts_with ("/usr/include/gtk-2.0/gtk/") then                Result := "gtk/gtk.h"
			elseif full_path.starts_with ("/usr/include/pango-1.0/pango/") then            Result := "pango/pango.h"
			else
				tmp := full_path.as_lower.substring_index ("include", 1)
				if tmp > 0 then
					Result := full_path.substring (tmp + 8, full_path.count)
				else
					Result := full_path
				end
			end
		ensure
			Result /= Void
		end

invariant
	is_system_source /= is_project_source
	source_file /= Void
	orig_source_file /= Void

end
