class
	OO_DIRECTORY

inherit
	OO_FILE

create
	make

feature {NONE}

	make (a_name: STRING_8)
		require
			a_name /= Void and then not a_name.is_empty
		do
			name := a_name
			files := create {TWO_WAY_LIST [OO_FILE]}.make
		end

feature {ANY}

	name: STRING_8

	files: LIST [OO_FILE]

	directories: LIST [OO_DIRECTORY]
		do
			Result := create {SORTED_TWO_WAY_LIST [OO_DIRECTORY]}.make
			across files as c loop
				if attached {OO_DIRECTORY} c.item as tmp then
					Result.extend (tmp)
				end
			end
		ensure
			Result /= Void
		end

	classes: LIST [OO_CLASS]
		do
			Result := create {SORTED_TWO_WAY_LIST [OO_CLASS]}.make
			across files as c loop
				if attached {OO_CLASS} c.item as tmp then
					Result.extend (tmp)
				end
			end
		ensure
			Result /= Void
		end

feature {ANY} -- directory handling

	get_directory (a_name: STRING_8): OO_DIRECTORY
		require
			a_name /= Void and then not a_name.is_empty
		do
			Result ?= get_file_of_type (a_name, True)
		end

	has_directory (a_name: STRING_8): BOOLEAN
		require
			a_name /= Void and then not a_name.is_empty
		do
			Result := get_directory (a_name) /= Void
		end

	create_directory (a_name: STRING_8): OO_DIRECTORY
		require
			a_name /= Void and then not a_name.is_empty
			not has_directory (a_name)
		do
			create Result.make (a_name)
			files.extend (Result)
		end

feature {ANY} -- class handling

	get_class (a_name: STRING_8): OO_CLASS
		require
			a_name /= Void and then not a_name.is_empty
		do
			Result ?= get_file_of_type (a_name, False)
		end

	has_class (a_class: OO_CLASS): BOOLEAN
		require
			a_class /= Void
		do
			Result := get_class (a_class.name) /= Void
		end

	add_class (a_class: OO_CLASS)
		require
			a_class /= Void
			not has_class (a_class)
		do
			files.extend (a_class)
		end

feature {NONE}

	get_file_of_type (a_name: STRING_8; a_type_directory: BOOLEAN): OO_FILE
		require
			a_name /= Void and then not a_name.is_empty
		do
			across
				files as c
			until
				Result /= Void
			loop
				if
					a_type_directory = attached {OO_DIRECTORY} c.item and then
					c.item.name.is_equal (a_name)
				then
					Result := c.item
				end
			end
		ensure
			Result /= Void and then a_type_directory implies attached {OO_DIRECTORY} Result
			Result /= Void and then not a_type_directory implies attached {OO_CLASS} Result
		end

invariant
	files /= Void

end
