class
	POINTER_ACCESS_INDEX_TRANSFORMER

inherit
	TRANSFORMATION_STEP
	AST_DEFAULT_VISITOR
		redefine
			end_visit_postfix_expression,
			end_visit_array_type,
			end_visit_generic_expression
		end

feature {ANY} -- TRANSFORMATION_STEP

	process_impl: BOOLEAN
		do
			transformer.tree.accept (Current)
			Result := True
		end

	description: STRING
		do
			Result := "Ensuring that the index for a pointer access is of type %"int%""
		end

feature {ANY} -- AST_DEFAULT_VISITOR

	end_visit_postfix_expression (node: AST_POSTFIX_EXPRESSION)
		do
			-- `arg[non_int_value]' -> `arg[(int)non_int_value]'
			if node.type = C_op_open_square_bracket then
				ensure_int_type (node.array_expression)
			end
		end

	end_visit_array_type (node: AST_ARRAY_TYPE)
		do
			-- `[non_int_value]' -> `[(int)non_int_value]'
			if node.dimension /= Void then
				ensure_int_type (node.dimension)
			end
		end

	end_visit_generic_expression (node: AST_GENERIC_EXPRESSION)
		do
			if
				node.is_additive_expression and then
				attached {AST_POINTER_TYPE} node.left.type_binding and then
				attached {AST_BASIC_TYPE} node.right.type_binding
			then
				ensure_int_type (node.right)
			end
		end

feature {NONE}

	ensure_int_type (array_index: AST_EXPRESSION)
		require
			array_index /= Void
		do
			if not (attached {AST_BASIC_TYPE} array_index.type_binding as tmp and then (tmp.type = Ast_bt_int or tmp.type = Ast_bt_long)) then
				replace_node (array_index, create {AST_CAST_EXPRESSION}.make (create {AST_BASIC_TYPE}.make (Ast_bt_int), array_index.ast_twin))
			end
		end

end
