class
	STRUCT_WRITER

inherit
	NAME_UTILITIES
	AST_UTILITIES

create
	make_node,
	make_oo

feature {NONE}

	make_node (a_node: AST_STRUCT_DECLARATION)
		require
			a_node /= Void
		do
			node := a_node
			create buffer.make_empty
		end

	make_oo (a_oo_class: OO_DATA_CLASS)
		require
			a_oo_class /= Void
		do
			oo_class := a_oo_class
			node := a_oo_class.node
			create buffer.make_empty
		end

feature {NONE}

	oo_class: OO_DATA_CLASS

	node: AST_STRUCT_DECLARATION

feature {NONE} -- convenience data

	superclasses: LIST [AST_STRUCT_DECLARATION]

	fields: LIST [AST_STRUCT_FIELD]

	is_union, has_bitfields, needs_initializers: BOOLEAN
		-- we need "make" if any of those 3 is True

feature {ANY}

	buffer: STRING

	write
		local
			l_max: INTEGER_32
		do
			superclasses := create {TWO_WAY_LIST [AST_STRUCT_DECLARATION]}.make
			if oo_class /= Void and then not oo_class.superclasses.is_empty then
				across oo_class.superclasses as c loop
					superclasses.extend (({OO_DATA_CLASS} #? c.item).node)
				end
			end

			fields := create {TWO_WAY_LIST [AST_STRUCT_FIELD]}.make
			across superclasses as c loop
				l_max := l_max.max (c.item.fields.count)
			end
			across node.fields as c loop
				if c.target_index > l_max then
					fields.extend (c.item)
				end
			end

			is_union := node.is_union
			has_bitfields := across fields as c some
					attached {AST_BITFIELD} c.item
				end
			needs_initializers := across fields as c some
					attached {AST_POINTER_TYPE} c.item.type or
					attached {AST_STRUCT_TYPE} c.item.type or
					attached {AST_ARRAY_TYPE} c.item.type
				end

			buffer.append ("class%N")
			buffer.append ("	" + eiffel_class_name (node.identifier) + "%N%N")

			write_inherit_clause

			buffer.append ("create%N")
			buffer.append ("	make,%N")
			buffer.append ("	make_shared%N%N")

			write_make
			write_fields
			write_setters
			write_copy_is_equal
			write_structure_info
			write_bitfield_helper

			buffer.append ("end%N")
		end

feature {NONE}

	write_inherit_clause
		local
			l_first: BOOLEAN
		do
			buffer.append ("inherit%N")
			if superclasses.is_empty then
				buffer.append ("	CE_CLASS%N")
				buffer.append ("		redefine%N")
				if is_union or has_bitfields or needs_initializers then
					buffer.append ("			make,%N")
				end
				buffer.append ("			copy,%N")
				buffer.append ("			is_equal")
				if has_bitfields then
					buffer.append (",%N")
					buffer.append ("			bitfield_helper%N")
				else
					buffer.append ("%N")
				end
				buffer.append ("		end%N")
			else
				across superclasses as c loop
					buffer.append ("	" + c.item.identifier + "%N")

					-- rename
					l_first := True
					node.fields.start
					across c.item.fields as c2 loop
						if
							attached c2.item.identifier as super_identifier and then
							attached node.fields.item.identifier as sub_identifier and then
							not super_identifier.is_equal (sub_identifier)
						then
							if l_first then
								buffer.append ("		rename%N")
								l_first := False
							else
								buffer.append (",%N")
							end
							buffer.append ("			" + super_identifier + " as " + sub_identifier)
						end
						node.fields.forth
					end
					if not l_first then
						buffer.append ("%N")
					end

					-- redefine
					buffer.append ("		redefine%N")
					if is_union or has_bitfields or needs_initializers then
						buffer.append ("			make,%N")
					end
					buffer.append ("			copy,%N")
					buffer.append ("			is_equal,%N")
					buffer.append ("			structure_info")
					if has_bitfields then
						buffer.append (",%N")
						buffer.append ("			bitfield_helper%N")
					else
						buffer.append ("%N")
					end

					buffer.append ("		end%N")
				end
			end
			buffer.append ("%N")
		end

	write_make
		local
			l_tmp_array: AST_ARRAY_TYPE
		do
			if is_union or has_bitfields or needs_initializers then
				buffer.append ("feature {NONE}%N%N")
				buffer.append ("	make%N")
				buffer.append ("		do%N")
				if is_union or has_bitfields then
					buffer.append ("			make_memory -- ")
					if is_union and has_bitfields then
						buffer.append ("union and has bitfields")
					elseif is_union then
						buffer.append ("union")
					else
						buffer.append ("has bitfields")
					end
					buffer.append ("; back with memory for correct semantics%N")
				else
					across fields as c loop
						if attached {AST_POINTER_TYPE} c.item.type or attached {AST_STRUCT_TYPE} c.item.type then
							buffer.append ("			create " + c.item.identifier + ".make%N")
						elseif attached {AST_ARRAY_TYPE} c.item.type as a then
							buffer.append ("			create " + c.item.identifier + ".make (<<")
							from
								l_tmp_array := a
							until
								l_tmp_array = Void
							loop
								if l_tmp_array /= a then
									buffer.append (", ")
								end
								if l_tmp_array.dimension /= Void then
									buffer.append (l_tmp_array.dimension.eiffel_code)
								else
									-- flexible array member at end of struct; no dimension needed
									buffer.append ("0")
								end
								l_tmp_array ?= l_tmp_array.type
							end
							buffer.append (">>)%N")
						end
					end
					if not superclasses.is_empty then
						buffer.append ("			Precursor%N")
					end
				end
				buffer.append ("		end%N%N")
			end
		end

	write_fields
		do
			buffer.append ("feature {ANY} -- fields%N%N")
			across fields as c loop
				if attached c.item.identifier as identifier then
					buffer.append ("	" + identifier + ": ")
					buffer.append (c.item.type.eiffel_code)
					if not attached {AST_ARRAY_TYPE} c.item.type then
						buffer.append (" assign set_" + identifier)
					end
					buffer.append ("%N%N")
				end
			end
		end

	write_setters
		local
			l_first: BOOLEAN
		do
			l_first := True
			across fields as c loop
				if
					attached c.item.identifier as identifier and
					not attached {AST_ARRAY_TYPE} c.item.type
				then
					if l_first then
						buffer.append ("feature {ANY} -- setters%N%N")
						l_first := False
					end

					buffer.append ("	set_" + identifier + " (a_" + identifier + ": ")
					buffer.append (c.item.type.eiffel_code)
					buffer.append (")%N")
					buffer.append ("		do%N")
					buffer.append ("			" + identifier + " := a_" + identifier + "%N")
					buffer.append ("			update_memory_field (%"" + identifier + "%")%N")
					buffer.append ("		end%N%N")
				end
			end
		end

	write_copy_is_equal
		local
			l_first: BOOLEAN
		do
			buffer.append ("feature {ANY}%N%N")

			buffer.append ("	copy (a_other: like Current)%N")
			buffer.append ("		do%N")
			if not superclasses.is_empty then
				buffer.append ("			Precursor (a_other)%N")
			end
			across fields as c loop
				if attached c.item.identifier as identifier then
					buffer.append ("			" + identifier + " := a_other." + identifier)
					if
						attached {AST_STRUCT_TYPE} c.item.type or
						attached {AST_ARRAY_TYPE} c.item.type
					then
						buffer.append (".twin")
					end
					buffer.append ("%N")
				end
			end
			if is_union or has_bitfields then
				buffer.append ("			create_memory_copy -- ")
				if is_union and has_bitfields then
					buffer.append ("union and has bitfields")
				elseif is_union then
					buffer.append ("union")
				else
					buffer.append ("has bitfields")
				end
				buffer.append ("; back with memory for correct semantics%N")
			end
			buffer.append ("		end%N%N")

			buffer.append ("	is_equal (a_other: like Current): BOOLEAN%N")
			buffer.append ("		do%N")
			buffer.append ("			Result :=%N")
			if
				superclasses.is_empty and
				(
					fields.is_empty or else
					across fields as c all c.item.identifier = Void end
				)
			then
				buffer.append ("True%N")
			else
				if not superclasses.is_empty then
					buffer.append ("				Precursor (a_other)")
					l_first := False
				else
					l_first := True
				end
				across fields as c loop
					if attached c.item.identifier as identifier then
						if l_first then
							l_first := False
						else
							buffer.append (" and%N")
						end
						buffer.append ("				")
						if
							attached {AST_POINTER_TYPE} c.item.type or
							attached {AST_STRUCT_TYPE} c.item.type or
							attached {AST_ARRAY_TYPE} c.item.type
						then
							buffer.append (identifier + ".is_equal (a_other." + identifier + ")")
						else
							buffer.append (identifier + " = a_other." + identifier)
						end
					end
				end
				buffer.append ("%N")
			end
			buffer.append ("		end%N%N")
		end

	write_structure_info
		local
			l_first: BOOLEAN
			l_tmp_array: AST_ARRAY_TYPE
		do
			buffer.append ("feature {NONE}%N%N")
			buffer.append ("	structure_info: CE_STRUCTURE%N")
			buffer.append ("		once (%"PROCESS%")%N")

			if is_union then
				buffer.append ("			create Result.make_union (<<%N")
			else
				buffer.append ("			create Result.make_struct (<<%N")
			end
			l_first := True
			across fields as c loop
				if attached c.item.identifier as identifier then
					if l_first then
						l_first := False
					else
						buffer.append (",%N")
					end
					buffer.append ("					create {CE_STRUCTURE_FIELD}.make")
					if attached {AST_ARRAY_TYPE} c.item.type then
						buffer.append ("_arrayfield")
					elseif attached {AST_BITFIELD} c.item then
						buffer.append ("_bitfield")
					end
					buffer.append (" (%"" + identifier + "%"")
					if attached {AST_ARRAY_TYPE} c.item.type as a then
						buffer.append (", <<")
						from
							l_tmp_array := a
						until
							l_tmp_array = Void
						loop
							if l_tmp_array /= a then
								buffer.append (", ")
							end
							if l_tmp_array.dimension /= Void then
								buffer.append (l_tmp_array.dimension.eiffel_code)
							else
								-- flexible array member at end of struct; no dimension needed
								buffer.append ("0")
							end
							l_tmp_array ?= l_tmp_array.type
						end
						buffer.append (">>")
					end
					buffer.append (")")
				end
			end
			if not l_first then
				buffer.append ("%N")
			end
			buffer.append ("				>>)%N")

			across fields as c loop
				if
					attached c.item.gcc_attribute_specifier as gas and then
					attached gas.find_attribute ("__aligned__") as a and then
					a.expressions.count = 1
				then
					buffer.append ("			Result.field_table[%"" + c.item.identifier + "%"].alignment := " + a.expressions.first.eiffel_code + "%N")
				end
			end

			if
				attached node.gcc_attribute_specifier as gas and then
				attached gas.find_attribute ("__aligned__") as a and then
				a.expressions.count = 1
			then
				buffer.append ("			Result.alignment := " + a.expressions.first.eiffel_code + "%N")
			end

			if not superclasses.is_empty then
				buffer.append ("			Result.set_superclasses (<<")
				across superclasses as c loop
					if c.target_index > 1 then
						buffer.append (", ")
					end
					buffer.append ("{" + eiffel_class_name (c.item.identifier) + "}")
				end
				buffer.append (">>)%N")
			end

			buffer.append ("			init_structure_info%N")
			buffer.append ("		end%N%N")
		end

	write_bitfield_helper
		local
			l_c_name: STRING_8
			l_tmp: BOOLEAN
		do
			if has_bitfields then
				if is_union then
					l_c_name := "union " + node.orig_c_identifier
				else
					l_c_name := "struct " + node.orig_c_identifier
				end
				buffer.append ("	bitfield_helper (a_operation: INTEGER_32; a_field_name, a_struct_pointer: POINTER; a_value: NATURAL_64): NATURAL_64%N")
				buffer.append ("		external%N")
				if node.source_info.is_project_source or node.orig_c_identifier.starts_with ("__anon") then
					buffer.append ("			%"C inline%"%N")
				else
					buffer.append ("			%"C inline use <" + node.source_info.source_file + ">%"%N")
				end
				buffer.append ("		alias%N")
				buffer.append ("			%"[%N")
				if node.source_info.is_project_source or node.orig_c_identifier.starts_with ("__anon") then
					buffer.append ("				" + l_c_name + " {%N")
					across node.fields as c loop
						buffer.append ("					" + c.item.c_code + ";%N")
					end
					buffer.append ("				};%N")
				end
				buffer.append ("				char *field_name = (char *)$a_field_name;%N")
				buffer.append ("				if($a_operation == 1) {%N")
				buffer.append ("					if(field_name == (char *)0) return sizeof(" + l_c_name + ");%N")
				l_tmp := False -- last was bitfield?
				across node.fields as c loop
					if attached {AST_BITFIELD} c.item then
						l_tmp := True
					elseif l_tmp and attached c.item.identifier as identifier then
						buffer.append ("					if(strcmp(field_name, %"" + identifier + "%") == 0) return offsetof(" + l_c_name + ", " + c.item.orig_c_identifier + ");%N")
						l_tmp := False
					end
				end
				buffer.append ("				} else {%N")
				buffer.append ("					" + l_c_name + " *p = (" + l_c_name + " *)$a_struct_pointer;%N")
				buffer.append ("					if($a_operation == 2) {%N")
				l_tmp := True -- first?
				across node.fields as c loop
					if attached {AST_BITFIELD} c.item and attached c.item.identifier as identifier then
						buffer.append ("						")
						if l_tmp then
							buffer.append ("     ")
							l_tmp := False
						else
							buffer.append ("else ")
						end
						buffer.append ("if(strcmp(field_name, %"" + identifier + "%") == 0) p->" + c.item.orig_c_identifier + " = (" + c.item.type.c_code + ")$a_value;%N")
					end
				end
				buffer.append ("					} else if($a_operation == 3) {%N")
				across node.fields as c loop
					if attached {AST_BITFIELD} c.item and attached c.item.identifier as identifier then
						buffer.append ("						if(strcmp(field_name, %"" + identifier + "%") == 0) return p->" + c.item.orig_c_identifier + ";%N")
					end
				end
				buffer.append ("					}%N")
				buffer.append ("				}%N")
				buffer.append ("			]%"%N")
				buffer.append ("		end%N%N")
			end
		end

invariant
	node /= Void
	buffer /= Void

end
