class
	VARARG_TRANSFORMER

inherit
	TRANSFORMATION_STEP
	AST_DEFAULT_VISITOR
		redefine
			end_visit_function_call,
			end_visit_builtin_va_arg_call,
			end_visit_function_declaration
		end

feature {ANY} -- TRANSFORMATION_STEP

	process_impl: BOOLEAN
		do
			transformer.tree.accept (Current)
			Result := True
		end

	description: STRING
		do
			Result := "Transforming varargs"
		end

feature {ANY} -- AST_DEFAULT_VISITOR

	end_visit_function_call (node: AST_FUNCTION_CALL)
		local
			cur: AST_NODE
			func_def: AST_FUNCTION_DECLARATION
		do
			if node.identifier_str /= Void then
				if node.identifier_str.is_equal ("__builtin_va_start") then
					from
						cur := node.parent
					until
						func_def /= Void
					loop
						if attached {AST_FUNCTION_DECLARATION} cur as tmp then
							func_def := tmp
						end
						cur := cur.parent
					end

					-- `va_start(argptr, num)' -> `argptr := create {CE_VA_LIST}.make (args, varargs_start_index)'
					replace_node (node, create {AST_EIFFEL_EXPRESSION}.make_mixed (
							<<node.parameters.first.ast_twin, " := create {CE_VA_LIST}.make (a_args, " + (func_def.parameter_list.parameters.count + 1).out + ")">>,
							create {AST_BASIC_TYPE}.make (Ast_bt_void)
						))
				elseif node.identifier_str.is_equal ("__builtin_va_end") then
					delete_node (node.parent)
				elseif node.identifier_str.is_equal ("__builtin_va_copy") then
					-- `va_copy(dest, src)' -> `dest = src'
					replace_node (node, create {AST_ASSIGNMENT_EXPRESSION}.make (node.parameters.i_th (1).ast_twin, node.parameters.i_th (2).ast_twin));
				end
			end
		end

	end_visit_builtin_va_arg_call (node: AST_BUILTIN_VA_ARG_CALL)
		do
			if attached {AST_BASIC_TYPE} node.type then
				-- `va_arg(argptr, int)' -> `argptr.integer_32_item'
				replace_node (node, create {AST_EIFFEL_EXPRESSION}.make_mixed (
						<<node.expression.ast_twin, "." + node.type.eiffel_code.as_lower, "_item">>,
						node.type
					))
			elseif attached {AST_POINTER_TYPE} node.type then
				-- `va_arg(argptr, int **)' -> `create {CE_POINTER [CE_POINTER [INTEGER_32]]}.make_cast (argptr.pointer_item)'
				replace_node (node, create {AST_PARENTHESIZED_EXPRESSION}.make (create {AST_EIFFEL_EXPRESSION}.make_mixed (
						<<"create {", node.type.ast_twin, "}.make_cast (", node.expression.ast_twin, ".pointer_item)">>,
						node.type
					)))
			elseif attached {AST_STRUCT_TYPE} node.type then
				-- `va_arg(argptr, foo)' -> `({foo} #? argptr.ce_class_item ({foo}))'
				replace_node (node, create {AST_PARENTHESIZED_EXPRESSION}.make (create {AST_EIFFEL_EXPRESSION}.make_mixed (
						<<"{", node.type.ast_twin, "} #? ", node.expression.ast_twin, ".ce_class_item ({", node.type.ast_twin, "})">>,
						node.type
					)))
			else
				log ("Unsupported type: " + node.type.generator)
			end
		end

	end_visit_function_declaration (node: AST_FUNCTION_DECLARATION)
		do
			if
				node.orig_c_identifier.is_equal ("__builtin_va_start") or
				node.orig_c_identifier.is_equal ("__builtin_va_arg") or
				node.orig_c_identifier.is_equal ("__builtin_va_end") or
				node.orig_c_identifier.is_equal ("__builtin_va_copy")
			then
				delete_node (node)
			end
		end

end
