class
	CLASS_UTILITIES

inherit
	NAME_UTILITIES

feature {ANY}

	get_class_prefix (caller_node, callee_node: AST_NODE): STRING
			-- If `caller_node' will end up in a different class than `callee_node',
			-- the required prefix is returned. `Void' otherwise.
		require
			caller_node /= Void
			callee_node /= Void
		local
			caller_node_decl, callee_node_decl: AST_EIFFEL_FEATURE
			caller_class, callee_class: OO_REGULAR_CLASS
		do
			caller_node_decl := get_enclosing_declaration (caller_node)
			callee_node_decl := get_enclosing_declaration (callee_node)
			if caller_node_decl = Void or callee_node_decl = Void then
				(create {EXCEPTIONS}).raise ("node not in tree or not child of a function declaration")
			end

			if callee_node_decl.source_info.is_system_source then
				-- no prefix needed; class will inherit from system class
			else
				if
					attached {AST_VARIABLE_DECLARATION} caller_node_decl as tmp and then
					not is_in_type_declaration (caller_node, tmp.type)
				then
					-- the call is done from a variable initializer which will end up in an own class
					caller_class := Void
				else
					caller_class := caller_node_decl.oo_feature.parent
				end
				callee_class := callee_node_decl.oo_feature.parent

				if caller_class /= callee_class then
					create Result.make_empty
					Result.append (callee_class.name)
					Result.append ("_inst.")
					Result.to_lower
					if Result.starts_with ("p_") then
						Result := Result.substring (3, Result.count)
					end
				end
			end
		end

feature {NONE}

	get_enclosing_declaration (node: AST_NODE): AST_EIFFEL_FEATURE
		require
			node /= Void
		local
			cur: AST_NODE
		do
			from
				cur := node
			until
				cur = Void or Result /= Void
			loop
				if attached {AST_EIFFEL_FEATURE} cur as tmp and then attached {AST_TRANSLATION_UNIT} tmp.parent then
					Result := tmp
				end
				cur := cur.parent
			end
		end

	is_in_type_declaration (node: AST_NODE; type: AST_TYPE): BOOLEAN
		require
			node /= Void
			type /= Void
		local
			cur: AST_NODE
		do
			from
				cur := node
			until
				cur = Void or Result
			loop
				if cur = type then
					Result := True
				end
				cur := cur.parent
			end
		end

end
