class
	DEPENDENCY_GRAPH [G]
inherit
	DEPENDENCY_GRAPH_FLAGS

create
	make

feature {NONE} -- Initialization

	make
			-- Initialization for `Current'.
		do
			create {ARRAYED_LIST [DEPENDENCY_GRAPH_ITEM [G]]} items.make (10)
		end

feature {NONE}

	items: LIST [DEPENDENCY_GRAPH_ITEM [G]]

feature {ANY}

	is_finalized: BOOLEAN

	insert_item (item: DEPENDENCY_GRAPH_ITEM [G])
		require
			not has_item_with_key (item.key)
			not is_finalized
		do
			items.extend (item)
			item.reference_number := items.count
		end

	update_item (key: G; identifier: STRING; flags: INTEGER)
		require
			has_item_with_key (key)
			not is_finalized
		local
			idx: INTEGER
		do
			idx := index_of (key)

			items[idx].identifier := identifier
			items[idx].add_flag (flags)
		end

	insert_dependency (dep_1, dep_2: G)
		require
			not is_finalized
		do
			insert_directed_dependency (dep_1, dep_2)
			insert_directed_dependency (dep_2, dep_1)
		end

	insert_directed_dependency (source_key, target_key: G)
			-- if target is bad then the source is also bad (a bad source has no effect on the target)
		require
			not is_finalized
		local
			source_idx: INTEGER
			target_idx: INTEGER
		do
			source_idx := index_of (source_key)
			if source_idx = 0 then
				insert_default_item (source_key)
				source_idx := items.count
			end

			target_idx := index_of (target_key)
			if target_idx = 0 then
				insert_default_item (target_key)
				target_idx := items.count
			end

			-- create a link from target to source (this is the correct direction!)
			items[target_idx].add_dep (items[source_idx])
		end

	has_item_with_key (key: G): BOOLEAN
		local
			idx: INTEGER
		do
			idx := index_of (key)

			Result := (idx > 0)
		end

	mark_bad (key: G; flag: INTEGER)
			-- marks the item `item' as bad, set flag as reason.
			-- if `item' is not already inserted, it is inserted
		require
			not is_finalized
		local
			i: INTEGER
		do
			i := index_of (key)

			if i = 0 then
				insert_default_item (key)
				i := items.count
			end

			items[i].add_flag (flag)
		end

	mark_all_bad (list: LIST [G]; flag: INTEGER)
			-- calls `mark_bad' for every item in list
		require
			not is_finalized
		do
			across
				list as c
			loop
				mark_bad (c.item, flag)
			end
		end

	good_items: LIST [G]
			-- returns a list with all keys not marked as bad and not have a dependency on a bad key
		require
			is_finalized
		do
			create {LINKED_LIST [G]} Result.make
			from
				items.start
			until
				items.after
			loop
				if not items.item.is_bad then
					Result.extend (items.item.key)
				end

				items.forth
			end
		end

	to_string: STRING
		require
			is_finalized
		do
			create Result.make_empty
			across
				items as i
			loop
				if not has_flag (i.item.flags, External_flag) then
					Result.append (i.item.to_string + "%N")
				end
			end

			Result.append ("%N")
		end

	get_statistics: STRING
		require
			is_finalized
		local
			extern, intern, good: INTEGER
			bad_ref, function, struct, union, pointer, cast, func_pointer, assignment, initializer, array, asm, other: INTEGER
			bad_marks: INTEGER
			flags: INTEGER
		do
			across
				items as i
			loop
				flags := i.item.flags

				if has_flag (flags, External_flag) then
					if not i.item.identifier.starts_with ("return value of") then
						extern := extern + 1
					end
				else
					if
						not i.item.identifier.starts_with ("temp cast expr") and then
						not i.item.identifier.starts_with ("???")
					then
						intern := intern + 1
						bad_marks := bad_marks + i.item.bad_marks

						if not i.item.is_bad then
							good := good + 1
						end

						if has_flag (flags, Bad_reference_flag) then
							bad_ref := bad_ref + 1
						end
						if has_flag (flags, External_function_flag) then
							function := function + 1
						end
						if has_flag (flags, Struct_member_flag) then
							struct := struct + 1
						end
						if has_flag (flags, Union_flag) then
							union := union + 1
						end
						if has_flag (flags, Pointer_flag) then
							pointer := pointer + 1
						end
						if has_flag (flags, Cast_flag) then
							cast := cast + 1
						end
						if has_flag (flags, Function_pointer_flag) then
							func_pointer := func_pointer + 1
						end
						if has_flag (flags, Assignment_flag) then
							assignment := assignment + 1
						end
						if has_flag (flags, Initializer_flag) then
							initializer := initializer + 1
						end
						if has_flag (flags, Array_flag) then
							array := array + 1
						end
						if has_flag (flags, Assembler_flag) then
							asm := asm + 1
						end
						if has_flag (flags, Other_flag) then
							other := other + 1
						end
					end
				end
			end

			create Result.make_empty
			Result.append ("Statistics:%N")
			Result.append ("-----------%N%N")

			Result.append ("external: " + extern.out + "%N")
			Result.append ("internal: " + intern.out + "%N")
			Result.append ("translatable: " + good.out + "%N%N")

			Result.append ("bad reference flags: " + bad_ref.out + " (" + (bad_ref * 100 / intern).out + "%%)%N")
			Result.append ("external function flags: " + function.out + " (" + (function * 100 / intern).out + "%%)%N")
			Result.append ("struct member flags: " + struct.out + " (" + (struct * 100 / intern).out + "%%)%N")
			Result.append ("union flags: " + union.out + " (" + (union * 100 / intern).out + "%%)%N")
			Result.append ("pointer flags: " + pointer.out + " (" + (pointer * 100 / intern).out + "%%)%N")
			Result.append ("cast flags: " + cast.out + " (" + (cast * 100 / intern).out + "%%)%N")
			Result.append ("function pointer flags: " + func_pointer.out + " (" + (func_pointer * 100 / intern).out + "%%)%N")
			Result.append ("assignment flags: " + assignment.out + " (" + (assignment * 100 / intern).out + "%%)%N")
			Result.append ("initializer flags: " + initializer.out + " (" + (initializer * 100 / intern).out + "%%)%N")
			Result.append ("array flags: " + array.out + " (" + (array * 100 / intern).out + "%%)%N")
			Result.append ("assembler flags: " + asm.out + " (" + (asm * 100 / intern).out + "%%)%N")
			Result.append ("other flags: " + other.out + " (" + (other * 100 / intern).out + "%%)%N")

			Result.append ("%N")
			Result.append ("total bad marks: " + bad_marks.out + "%N")
			Result.append ("average bad marks: " + (bad_marks / intern).out + "%N%N")
		end

	finalize
		require
			not is_finalized
		local
			idx: INTEGER
			queue: LINKED_QUEUE [INTEGER]
		do
			-- update marks
			create queue.make

			across
				items as i
			loop
				if i.item.is_bad then
					queue.put (i.item.reference_number)
				end
			end

			-- bfs on the graph
			from
			until
				queue.is_empty
			loop
				idx := queue.item
				queue.remove

				across
					items[idx].dependencies as c
				loop
					if not c.item.is_bad then
						queue.put (c.item.reference_number)
						c.item.add_flag (bad_reference_flag)
					end
				end
			end

			is_finalized := True
		ensure
			is_finalized
		end

feature -- Implementation

	index_of (key: G): INTEGER
		local
			pos: INTEGER
			found: BOOLEAN
		do
			Result := 0

			from
				pos := 1
				found := False
			until
				pos > items.count or found
			loop
				if items[pos].key.is_equal (key) then
					found := True
					Result := pos
				end

				pos := pos + 1
			end
		end

	insert_default_item (key: G)
		require
			not is_finalized
			not has_item_with_key (key)
		local
			new_item: DEPENDENCY_GRAPH_ITEM [G]
		do
			create new_item.make (key, "???", 0)
			insert_item (new_item)
		end

end
