deferred class
	ANALYZER

inherit
	AST_DEFAULT_VISITOR
		redefine
			visit_function_declaration,
			end_visit_function_declaration
		end
	ANTLR_AST_TYPE_CONSTANTS
	AST_UTILITIES
	DEPENDENCY_GRAPH_FLAGS

feature -- Initialisation

	make
		do
			create collector
			setup_collector

			is_extern_function_declaration := False
		end

feature -- Status report

	is_null_check (node: AST_GENERIC_EXPRESSION): BOOLEAN
		local
			left_cast, right_cast: AST_CAST_EXPRESSION
		do
			if  is_equality_check (node)  then
				left_cast ?= node.left
				right_cast ?= node.right

				-- check if one side is `((...)0)'
				if
					is_null_pointer (left_cast)
				then
					-- visit right child
					right_cast.expression.accept (Current)
					Result := True
				elseif
					is_null_pointer (right_cast)
				then
					-- visit left child
					left_cast.expression.accept (Current)
					Result := True
				end
			end
		end

	is_equality_check (node: AST_GENERIC_EXPRESSION): BOOLEAN
		do
			if -- check for `(unsigned int)(...) == (unsignet int)(...)' or inequality of the same form
				node.is_equality_expression and then
				attached {AST_CAST_EXPRESSION} node.left as left_cast and then
				attached {AST_BASIC_TYPE} left_cast.type as left_bt and then
				left_bt.type = ast_bt_unsigned_int and then
				attached {AST_CAST_EXPRESSION} node.right as right_cast and then
				attached {AST_BASIC_TYPE} right_cast.type as right_bt and then
				right_bt.type = ast_bt_unsigned_int
			then
				Result := True
			end
		end

	is_null_pointer (node: AST_CAST_EXPRESSION): BOOLEAN
		do
			if
				attached {AST_DECIMAL_CONSTANT} node.expression as const and then
				const.value.is_equal ("0")
			then
				Result := True
			end
		end

feature -- redefined methods

	visit_function_declaration (node: AST_FUNCTION_DECLARATION): BOOLEAN
		do
			current_function_declaration := node
			if node.is_extern then
				is_extern_function_declaration := True
			end

			Result := True
		end

	end_visit_function_declaration (node: AST_FUNCTION_DECLARATION)
		do
			current_function_declaration := Void
			is_extern_function_declaration := False
		end

feature {NONE} -- Collect type objects in subtrees

	collector: AST_AGENT_VISITOR

	collected_types: LINKED_LIST [AST_TYPE]

	setup_collector
		do
			collector.visit_array_type_agent := agent (node: AST_ARRAY_TYPE): BOOLEAN
					do
						Result := collector_visit_type (node)
					end
			collector.visit_cast_expression_agent := agent (node: AST_CAST_EXPRESSION): BOOLEAN
					do
						Result := not is_null_pointer (node)
					end
			collector.visit_function_call_agent := agent (node: AST_FUNCTION_CALL): BOOLEAN
					do
						if
							node.declaration_binding /= Void and then
							collect_type (node.declaration_binding.return_type)
						then
							collected_types.extend (node.declaration_binding.return_type)
						end
						Result := False
					end
			collector.visit_function_declaration_agent := agent (node: AST_FUNCTION_DECLARATION): BOOLEAN
					do
						Result := not node.is_extern
					end
			collector.visit_pointer_type_agent := agent (node: AST_POINTER_TYPE): BOOLEAN
					do
						Result := collector_visit_type (node)
					end
			collector.visit_postfix_expression_agent := agent (node: AST_POSTFIX_EXPRESSION): BOOLEAN
					do
						if
							node.declaration_binding /= Void and then
							collect_type (node.declaration_binding.type)
						then
							collected_types.extend (node.declaration_binding.type)
						end
						Result := True
					end
			collector.visit_variable_expression_agent := agent (node: AST_VARIABLE_EXPRESSION): BOOLEAN
					do
						if collect_type (node.declaration_binding.type) then
							collected_types.extend (node.declaration_binding.type)
						end
						Result := True
					end
		end

	collector_visit_type (type: AST_TYPE): BOOLEAN
		do
			if collect_type (type) then
				collected_types.extend (type)
			end
			Result := True
		end

	collect_in_sub_tree (node: AST_NODE): LINKED_LIST [AST_TYPE]
			-- collects all ast_type objects in the sub_tree `node' for which `collect_type' returns True
		do
			create collected_types.make
			node.accept (collector)
			Result := collected_types
		end

	collect_in_list (list: LIST [ANY]) : LINKED_LIST [AST_TYPE]
			-- collects all ast_type objects in the list `list' for which `collect_type' returns True
		do
			create Result.make

			across
				list as c
			loop
				if attached {AST_NODE} c.item as node then
					Result.append (collect_in_sub_tree (node))
				end
			end
		end

	collect_type (type: AST_TYPE): BOOLEAN
		require
			type /= Void
		deferred
		end

feature {NONE} -- Implementation

	is_extern_function_declaration: BOOLEAN

	current_function_declaration: AST_FUNCTION_DECLARATION

end
