class
	AST_VIEWER

inherit
	EV_TITLED_WINDOW
		redefine
			initialize,
			show
		end
	VISION2_GC_WORKAROUND

feature {ANY}

	tree: EV_TREE

	show
		do
			from
				tree.start
			until
				tree.after
			loop
				add_mouse_listener (tree.item)
				tree.forth
			end
			Precursor
		end

feature {NONE}

	menu: EV_MENU
	mi_expand_all: EV_MENU_ITEM
	mi_show_source: EV_MENU_ITEM
	mi_show_type_binding: EV_MENU_ITEM

	initialize
		local
			main_container: EV_VERTICAL_BOX
		do
			Precursor {EV_TITLED_WINDOW}

			create main_container
			extend (main_container)

			create menu
			create mi_expand_all.make_with_text_and_action ("Expand All", agent expand_all_requested)
			menu.extend (mi_expand_all)
			create mi_show_source.make_with_text_and_action("Show Source", agent show_source_requested)
			menu.extend (mi_show_source)
			create mi_show_type_binding.make_with_text_and_action("Show Type Binding", agent show_type_binding_requested)
			menu.extend (mi_show_type_binding)

			create tree
			main_container.extend (tree)

			close_request_actions.extend (agent close_request)

			set_title ("TreeViewer")
			set_size (800, 600)
		end

	add_mouse_listener (tree_item: EV_TREE_NODE)
		do
			tree_item.pointer_button_press_actions.extend (agent menu_request (tree_item, ?, ?, ?, ?, ?, ?, ?, ?))
			from
				tree_item.start
			until
				tree_item.after
			loop
				add_mouse_listener (tree_item.item)
				tree_item.forth
			end
		end

	menu_request (selected_item: EV_TREE_NODE; x, y, button: INTEGER; x_tilt, y_tilt, pressure: DOUBLE; e_screen_x, e_screen_y: INTEGER)
		do
			if button = 3 then
				selected_item.enable_select
				if selected_item.is_expandable then
					mi_expand_all.enable_sensitive
				else
					mi_expand_all.disable_sensitive
				end
				if attached {AST_NODE} selected_item.data then
					mi_show_source.enable_sensitive
				else
					mi_show_source.disable_sensitive
				end
				if attached {AST_EXPRESSION} selected_item.data then
					mi_show_type_binding.enable_sensitive
				else
					mi_show_type_binding.disable_sensitive
				end
				menu.show_at (Void, e_screen_x, e_screen_y)
			end
		end

	expand_all_requested
		do
			expand_all (tree.selected_item)
		end

	show_source_requested
		local
			source_viewer: SOURCE_VIEWER
		do
			create source_viewer
			protect_window_from_gc (source_viewer)
			source_viewer.source_code.set_text (({AST_NODE} #? tree.selected_item.data).c_code)
			source_viewer.show
		end

	show_type_binding_requested
		local
			l_message_dialog: EV_INFORMATION_DIALOG
		do
			create l_message_dialog.make_with_text (({AST_EXPRESSION} #? tree.selected_item.data).type_binding.eiffel_code)
			l_message_dialog.set_title ("Type Binding")
			l_message_dialog.show_modal_to_window (Current)
		end

	expand_all (cur: EV_TREE_NODE)
		do
			if cur.is_expandable then
				cur.expand
				from
					cur.start
				until
					cur.after
				loop
					expand_all (cur.item)
					cur.forth
				end
			end
		end

	close_request
		do
			destroy
		end

end
