class
	CE_UTILITIES

feature {ANY} -- string utilities

	ce_string (a_s: STRING_8): CE_ARRAY [INTEGER_8]
		require
			a_s /= Void
		local
			l_i: INTEGER_32
		do
			Result := static_string_storage[a_s]
			if Result = Void then
				create Result.make (<<a_s.count + 1>>)
				from
					l_i := 1
				until
					l_i > a_s.count
				loop
					Result[l_i - 1] := a_s[l_i].code.to_integer_8
					l_i := l_i + 1
				end
				static_string_storage[a_s] := Result
			end
		ensure
			Result /= Void
		end

	eif_string (a_p: CE_POINTER [INTEGER_8]): STRING
		require
			a_p /= Void
		do
			create Result.make_from_c (a_p.c_pointer)
		ensure
			Result /= Void
		end

feature {ANY} -- ce-type utilities

	Ce_type_boolean,
	Ce_type_integer_8,
	Ce_type_integer_16,
	Ce_type_integer_32,
	Ce_type_integer_64,
	Ce_type_natural_8,
	Ce_type_natural_16,
	Ce_type_natural_32,
	Ce_type_natural_64,
	Ce_type_real_32,
	Ce_type_real_64,
	Ce_type_real_96,
	Ce_type_void,
	Ce_type_va_list,
	Ce_type_pointer,
	Ce_type_array,
	Ce_type_class: INTEGER_32 = unique

	ce_type (a_eif_type: TYPE [ANY]): INTEGER_32
		require
			a_eif_type /= Void
		do
			    if a_eif_type = {BOOLEAN}    then Result := Ce_type_boolean
			elseif a_eif_type = {INTEGER_8}  then Result := Ce_type_integer_8
			elseif a_eif_type = {INTEGER_16} then Result := Ce_type_integer_16
			elseif a_eif_type = {INTEGER_32} then Result := Ce_type_integer_32
			elseif a_eif_type = {INTEGER_64} then Result := Ce_type_integer_64
			elseif a_eif_type = {NATURAL_8}  then Result := Ce_type_natural_8
			elseif a_eif_type = {NATURAL_16} then Result := Ce_type_natural_16
			elseif a_eif_type = {NATURAL_32} then Result := Ce_type_natural_32
			elseif a_eif_type = {NATURAL_64} then Result := Ce_type_natural_64
			elseif a_eif_type = {REAL_32}    then Result := Ce_type_real_32
			elseif a_eif_type = {REAL_64}    then Result := Ce_type_real_64
			elseif a_eif_type = {REAL_96}    then Result := Ce_type_real_96
			elseif a_eif_type = {ANY}        then Result := Ce_type_void
			elseif a_eif_type = {CE_VA_LIST} then Result := Ce_type_va_list
			elseif attached {TYPE [CE_ARRAY [ANY]]} a_eif_type   then Result := Ce_type_array
			elseif attached {TYPE [CE_POINTER [ANY]]} a_eif_type then Result := Ce_type_pointer
			elseif attached {TYPE [CE_CLASS]} a_eif_type         then Result := Ce_type_class
			else
				(create {EXCEPTIONS}).raise (a_eif_type.name)
			end
		end

feature {ANY} -- alignment utilities

	ce_alignof (a_eif_type: TYPE [ANY]): NATURAL_32
		require
			a_eif_type /= Void
		local
			l_tmp_type: TYPE [ANY]
			l_tmp_class: CE_CLASS
			l_internal: INTERNAL
		do
			inspect ce_type (a_eif_type)
			when Ce_type_void, Ce_type_boolean, Ce_type_integer_8, Ce_type_natural_8 then
				Result := 1
			when Ce_type_integer_16, Ce_type_natural_16 then
				Result := 2
			when
				Ce_type_integer_32, Ce_type_integer_64,
				Ce_type_natural_32, Ce_type_natural_64,
				Ce_type_real_32, Ce_type_real_64, Ce_type_real_96,
				Ce_type_pointer, Ce_type_va_list
			then
				Result := 4
			when Ce_type_array then
				from
					l_tmp_type := a_eif_type.generic_parameter_type (1)
				until
					not attached {TYPE [CE_ARRAY [ANY]]} l_tmp_type
				loop
					l_tmp_type := l_tmp_type.generic_parameter_type (1)
				end
				Result := ce_alignof (l_tmp_type)
			when Ce_type_class then
				create l_internal
				l_tmp_class ?= l_internal.new_instance_of (a_eif_type.type_id)
				if l_tmp_class.structure_info.alignment /= 0 then
					Result := l_tmp_class.structure_info.alignment.to_natural_32
				elseif
					attached l_tmp_class.structure_info.fields as f and then
					f.count > 0
				then
					if f[0].alignment /= 0 then
						Result := f[0].alignment.to_natural_32
					else
						Result := ce_alignof (l_internal.type_of_type (f[0].eif_type))
					end
				else
					Result := 1
				end
			end
		ensure
			Result = 1 or Result = 2 or Result = 4
		end

feature {ANY} -- REAL_96 utilities

	Real_96_bytes: INTEGER_32 = 12

	real_96_from_integer_8 (a_value: INTEGER_8): REAL_96
		do
			a_value.do_nothing -- bug 17924
			Result.set_from_integer_8 (a_value)
		end

	real_96_from_integer_16 (a_value: INTEGER_16): REAL_96
		do
			a_value.do_nothing -- bug 17924
			Result.set_from_integer_16 (a_value)
		end

	real_96_from_integer_32 (a_value: INTEGER_32): REAL_96
		do
			a_value.do_nothing -- bug 17924
			Result.set_from_integer_32 (a_value)
		end

	real_96_from_integer_64 (a_value: INTEGER_64): REAL_96
		do
			a_value.do_nothing -- bug 17924
			Result.set_from_integer_64 (a_value)
		end

	real_96_from_natural_8 (a_value: NATURAL_8): REAL_96
		do
			a_value.do_nothing -- bug 17924
			Result.set_from_natural_8 (a_value)
		end

	real_96_from_natural_16 (a_value: NATURAL_16): REAL_96
		do
			a_value.do_nothing -- bug 17924
			Result.set_from_natural_16 (a_value)
		end

	real_96_from_natural_32 (a_value: NATURAL_32): REAL_96
		do
			a_value.do_nothing -- bug 17924
			Result.set_from_natural_32 (a_value)
		end

	real_96_from_natural_64 (a_value: NATURAL_64): REAL_96
		do
			a_value.do_nothing -- bug 17924
			Result.set_from_natural_64 (a_value)
		end

	real_96_from_real_32 (a_value: REAL_32): REAL_96
		do
			a_value.do_nothing -- bug 17924
			Result.set_from_real_32 (a_value)
		end

	real_96_from_real_64 (a_value: REAL_64): REAL_96
		do
			a_value.do_nothing -- bug 17924
			Result.set_from_real_64 (a_value)
		end

feature {ANY} -- hex floating point utilities

	hex_to_real_32 (a_str: STRING_8): REAL_32
		local
			l_c_str: C_STRING
		do
			create l_c_str.make (a_str)
			Result := c_hex_to_real_32 (l_c_str.item)
		end

	hex_to_real_64 (a_str: STRING_8): REAL_64
		local
			l_c_str: C_STRING
		do
			create l_c_str.make (a_str)
			Result := c_hex_to_real_64 (l_c_str.item)
		end

	hex_to_real_96 (a_str: STRING_8): REAL_96
		local
			l_c_str: C_STRING
		do
			create l_c_str.make (a_str)
			c_hex_to_real_96 (Result.c_pointer, l_c_str.item)
		end

feature {NONE}

	static_string_storage: HASH_TABLE [CE_ARRAY [INTEGER_8], STRING_8]
		once ("PROCESS")
			create Result.make (16)
		end

	c_hex_to_real_32 (a_str: POINTER): REAL_32
		external
			"C inline use <stdlib.h>"
		alias
			"return strtof($a_str, 0);"
		end

	c_hex_to_real_64 (a_str: POINTER): REAL_64
		external
			"C inline use <stdlib.h>"
		alias
			"return strtod($a_str, 0);"
		end

	c_hex_to_real_96 (a_res, a_str: POINTER)
		external
			"C inline use <stdlib.h>"
		alias
			"*(long double *)$a_res = strtold($a_str, 0);"
		end

end
